"""

This directory contains PDF documents from the DUDE 2023 shared
task and OCR outputs that can help evaluate models. Note it is not
required to use the attached OCRs, and we do not guarantee their
correctness.

Specifically, available OCRs include outputs of:
- Azure Cognitive Services (3.2),
- Amazon Textract (2018-06-27),
- Tesseract (5.3.0).

The output of Azure and Amazon OCRs was obtained from the original
PDF files. Since Tesseract does not support PDF inputs, we converted
them to TIFFs (200 dpi) before running the process. There were three
files where this process failed due to format limitations.

In addition to software-specific outputs, we provide OCRs in the
unified form introduced by the authors of DUE Benchmark. Please find
a toy reader of these below.

Example:
> read_document('fff498b60f0224b08d76276ac1c22abc')

Last update: 2023-01-23 (v1.0)

"""

import json
from typing import Dict, List, Tuple, Literal

from pdf2image import convert_from_path
from PIL.Image import Image


def read_document(
        file_id: str,
        subset: Literal['train', 'val', 'test'] = 'train',
        ocr_engine: Literal['Azure', 'Amazon', 'Tesseract'] = 'Azure'
    ) -> Tuple[List[Image], Dict]:

    # Read images of individual pages from PDF
    images: List[Image] = convert_from_path(f'PDF/{subset}/{file_id}.pdf')

    # Read OCR results in DUE format
    with open(f'OCR/{ocr_engine}/{file_id}_due.json') as ins:
        data: Dict = json.load(ins)

    # Validate if OCRs and convert_from_path have compatible
    # coordinates
    page_bboxes = data['structures']['pages']['positions']
    assert len(images) == len(page_bboxes)
    assert [[0, 0, *i.size] for i in images] == page_bboxes

    return (images, data)
